from collections import OrderedDict

from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtGui import QColor,QStandardItem, QStandardItemModel
from PyQt5.QtWidgets import (
    QAbstractItemView, QApplication, QColorDialog, QComboBox, QDoubleSpinBox,
    QGridLayout, QStyledItemDelegate, QSpinBox, QTableView, QLineEdit, QWidget
)

class Colors():

    colors = OrderedDict()
    colorNames = OrderedDict()

    for colorName in QColor.colorNames():
        color = QColor(colorName)
        rgb = color.getRgb()[:3]
        colors[rgb] = color
        colorNames[rgb] = colorName

    @classmethod
    def updateColor(cls, rgbstr):
        try:
            rgb, name = cls.str2rgb(rgbstr)
        except ValueError as e:
            print('Colors.updateColor: Invalid color value %s: %s' % (rgbstr, e))
            raise
        else:
            color = QColor(*rgb)
            if rgb not in cls.colors:
                cls.colors[rgb] = color
                cls.colors.move_to_end(rgb, last = False)
            if name:
                cls.colorNames[rgb] = name
            return cls.colorLabel(color), color

    @classmethod
    def lookupColor(cls, color):
        rgb = color.getRgb()[:3]
        name = cls.colorNames.get(rgb, '')
        return rgb, name

    @classmethod
    def colorLabel(cls, color):
        rgb, name = cls.lookupColor(color)
        label = cls.rgb2str(rgb)
        if name:
            label += " (%s)" % name
        return label

    @staticmethod
    def rgb2str(rgb):
        return ','.join(map(str, rgb))

    @staticmethod
    def rgb2name(rgb):
        return '#%02x%02x%02x' % (rgb)

    @staticmethod
    def str2rgb(rgbstr):
        if not rgbstr:
            raise ValueError('Invalid RGB tuple: %r' % rgbstr)
        name = ''
        if ' ' in rgbstr:
            rgbstr, name = rgbstr.split(' ', 1)
        # convert r,g,b string to int tuple
        rgb = tuple(map(int, rgbstr.split(',')))[:3]
        # sanity checks
        if len(rgb) != 3:
            raise ValueError('Invalid RGB tuple length: %s' % str(rgb))
        if max(rgb) > 256:
            raise ValueError('Invalid values in RGB tuple: %s' % str(rgb))
        if min(rgb) < 0:
            raise ValueError('Invalid values in RGB tuple: %s' % str(rgb))
        if name and name[0] == '(' and name[-1] == ')':
            name = name[1:-1]
        #print('str2rgb(%s): %s "%s"' % (rgbstr, rgb, name))
        return rgb, name


class ModelEditor():

    def createEditor(self, parent, option, index):
        editor = QLineEdit(parent)
        editor.setFrame(False)
        return editor

    def setData(self, editor, value):
        editor.setText(value)

    def setEditorData(self, editor, index):
        self.setData(editor, index.data(Qt.EditRole))

    def getData(self, editor):
        return editor.text()

    def setModelData(self, editor, model, index):
        model.setData(index, self.getData(editor), Qt.EditRole)

    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)


class IntRange(ModelEditor):

    def __init__(self, start = None, stop = None, step = None):
        self.start = start
        self.stop = stop
        self.step = step

    def createEditor(self, parent, option, index):
        editor = QSpinBox(parent)
        editor.setFrame(False)
        if self.start is not None:
            editor.setMinimum(self.start)
        if self.stop is not None:
            editor.setMaximum(self.stop)
        if self.step is not None:
            editor.setSingleStep(self.step)
        return editor

    def setData(self, editor, value):
        editor.setValue(value)

    def getData(self, editor):
        editor.interpretText()
        return editor.value()


class FloatRange(IntRange):

    def __init__(self, start = None, stop = None, step = None, decimals = None):
        self.start = start
        self.stop = stop
        self.step = step
        self.decimals = decimals

    def createEditor(self, parent, option, index):
        editor = QDoubleSpinBox(parent)
        editor.setFrame(False)
        if self.start is not None:
            editor.setMinimum(self.start)
        if self.stop is not None:
            editor.setMaximum(self.stop)
        if self.step is not None:
            editor.setSingleStep(self.step)
        if self.decimals is not None:
            editor.setDecimals(self.decimals)
        return editor


class Choice(ModelEditor):

    def __init__(self, *choices):
        self.choices = choices

    def createEditor(self, parent, option, index):
        editor = QComboBox(parent)
        editor.setFrame(False)
        editor.setAutoFillBackground(True)
        for i, choice in enumerate(self.choices):
            editor.insertItem(i, choice)
        return editor

    def setData(self, editor, value):
        editor.setCurrentIndex(self.getIndex(value))

    def getData(self, editor):
        return editor.currentText()

    def getIndex(self, value):
        return self.choices.index(value)


class Bool(Choice):

    def __init__(self, choices = None):
        if choices is not None:
            self.choices = choices
        else:
            self.choices = ('false', 'true')


class Text(ModelEditor):
    pass


class Color(ModelEditor):

    def __init__(self):
        self.chooseColor = 'Choose Color...'
        self.lastValue = None
        self.lastColor = None
        self.itemView = None

    def createEditor(self, parent, option, index):
        self.itemView = itemView = parent.parent()
        print(itemView)
        editor = QComboBox(parent)
        editor.setFrame(False)
        editor.setAutoFillBackground(True)
        editor.setEditable(True)
        for i, (rgb, color) in enumerate(Colors.colors.items()):
            label = Colors.colorLabel(color)
            editor.insertItem(i, label)
            editor.setItemData(i, color, Qt.DecorationRole)
        # add color chooser item
        editor.insertSeparator(i+1)
        editor.insertItem(i+2, self.chooseColor)
        editor.currentIndexChanged[str].connect(self.currentTextChanged)
        return editor

    def currentTextChanged(self, value):
        if value == self.chooseColor:
            label = Colors.colorLabel(self.lastColor)
            print('selectColorDialog:', label)
            self.itemView.selectColorDialog.emit()
        else:
            try:
                label, color = Colors.updateColor(value)
            except ValueError:
                return
            else:
                print('currentTextChanged: %s, %s' % (label, color))
                self.lastColor = color
                self.lastValue = label

    def setData(self, editor, value):
        print('Color.setData(%s)' % value)
        try:
            label, color = Colors.updateColor(value)
        except ValueError:
            return
        idx = editor.findText(label)
        if idx < 0:
            editor.insertItem(0, label)
            idx = 0
        editor.setCurrentIndex(idx)
        self.lastValue = label
        self.lastColor = color
        if 0:
            editor.showPopup()

    def getData(self, editor):
        value = editor.currentText()
        print('Color.getData(%s)' % value)
        if value == self.chooseColor and self.lastValue:
            value = self.lastValue
            print('Color.getData(%s)' % value)
        try:
            label, color = Colors.updateColor(value)
        except ValueError:
            return
        self.lastValue = label
        self.lastColor = color
        print('Color.getData(%s) -> %s' % (value, label))
        return label


class ItemDelegate(QStyledItemDelegate):

    def createEditor(self, parent, option, index):
        cls = index.data(Qt.UserRole)
        if isinstance(cls, ModelEditor):
            return cls.createEditor(parent, option, index)
        else:
            return super(ItemDelegate, self).createEditor(parent, option, index)

    def setEditorData(self, editor, index):
        cls = index.data(Qt.UserRole)
        if isinstance(cls, ModelEditor):
            cls.setEditorData(editor, index)
        else:
            super(ItemDelegate, self).setEditorData(editor, index)

    def setModelData(self, editor, model, index):
        cls = index.data(Qt.UserRole)
        if isinstance(cls, ModelEditor):
            cls.setModelData(editor, model, index)
        else:
            super(ItemDelegate, self).setModelData(editor, model, index)

    def updateEditorGeometry(self, editor, option, index):
        cls = index.data(Qt.UserRole)
        if isinstance(cls, ModelEditor):
            cls.updateEditorGeometry(editor, option, index)
        else:
            super(ItemDelegate, self).updateEditorGeometry(editor, option, index)


class ColorItem(QStandardItem):

    def __init__(self, color = None):
        super(ColorItem, self).__init__()
        if color is not None:
            self.setData(color)

    def setData(self, value, role = Qt.EditRole):
        if role == Qt.EditRole:
            try:
                rgb, name = Colors.str2rgb(value)
            except ValueError as e:
                print('Invalid color value: %s' % e)
            else:
                color = QColor(*rgb)
                # display color icon
                super(ColorItem, self).setData(color, Qt.DecorationRole)
                # replace color label
                value = Colors.colorLabel(color)
        super(ColorItem, self).setData(value, role)

    def type(self):
        return QStandardItem.UserType + 234


class TableView(QTableView):

    selectColorDialog = pyqtSignal()

    def __init__(self, model, parent = None):
        super(TableView, self).__init__(parent)
        self.setModel(model)
        self.selectColorDialog.connect(self.colorDialog)

    def colorDialog(self):
        index = self.currentIndex()
        print('colorDialog:', index, index.data(Qt.EditRole))
        label, color = Colors.updateColor(index.data(Qt.EditRole))
        color = QColorDialog.getColor(color, self)
        if color.isValid():
            rgb, name = Colors.lookupColor(color)
            label = Colors.colorLabel(color)
            print(label)
            model = index.model()
            print(model.data(index))
            print(index.row(), index.column())
            model.setData(index, label, Qt.EditRole)



class Window(QWidget):

    def __init__(self, parent = None):
        super(Window, self).__init__(parent)
        self.setWindowTitle("Multiplex Item Delegate")

        data = [
            # label, value, class, args
            [ 'intrange', 0, IntRange, (-10, 10, 1)],
            [ 'floatrange', 0.0, FloatRange, (-1.0, 1.0, 0.1, 2)],
            [ 'choice', 'one', Choice, ('one', 'two',  'three', 'four', 'hundred')],
            [ 'bool', 'false', Bool, ()],
            [ 'text', 'a text', Text, ()],
            [ 'color', '0,139,139', Color, ()],
            [ 'color', '255,215,0', Color, ()],
        ]

        # create model, view, and delegate
        self.model = model = QStandardItemModel(len(data), 2)
        tableView = TableView(model)

        delegate = ItemDelegate()
        tableView.setItemDelegate(delegate)
        tableView.horizontalHeader().setStretchLastSection(True)
        if 1:
            tableView.setEditTriggers(
                QAbstractItemView.EditKeyPressed |
                QAbstractItemView.CurrentChanged |
                QAbstractItemView.DoubleClicked |
                QAbstractItemView.SelectedClicked)
        else:
            tableView.setEditTriggers(
                QAbstractItemView.EditKeyPressed |
                QAbstractItemView.DoubleClicked |
                QAbstractItemView.SelectedClicked)

        # fill model with data
        for row, (label, value, cls, args) in enumerate(data):
            # column 0
            model.setItem(row, 0, QStandardItem(label))
            # column 1
            if cls == Color:
                item = ColorItem()
            else:
                item = QStandardItem()
            item.setData(value, Qt.EditRole)
            item.setData(cls(*args), Qt.UserRole)
            model.setItem(row, 1, item)

        # finalize window
        layout = QGridLayout()
        layout.addWidget(tableView, 0, 0)
        self.setLayout(layout)
        self.resize(400, 300)

    def result(self):
        model = self.model
        for row in range(model.rowCount()):
            label = model.item(row, 0).data(Qt.EditRole)
            value = model.item(row, 1).data(Qt.EditRole)
            print('[%s] %s: %s' % (row, label, value))


if __name__ == '__main__':
    import sys
    app = QApplication(sys.argv)
    window = Window()
    window.show()
    ret = app.exec_()
    window.result()
    sys.exit(ret)

